*     To load an ASCII file right to the RAM in a Dragon we will need:
*     Ask DW4 to send chunks of 256 bytes (sectors) of the requested named_object
*     Put in a RAM buffer a whole Basic sentence (till a $0D is read)
*     Then programm will make $A5-$A6 point to this buffer area and will call $83A3
*     that, in turn, has been copied here to rebuild the stack it re-initializes 
*		on every call we make.
*     On return, the program will fill the buffer again with next basic line untill the end of file
*     
*     PROBLEM:
*     The files we want lo load, for shure will not be %256 bytes long!
*     So they will have to be padded so that DW4 doesn't refuse to open them!
*
*     WARNING:
*     It is known that Dragon refuses to receive a Basic line longer than 249 bytes (cuts it to that length)
*     But you can later Edit that Line and add some chars
*
* LDA17S.DRM
*
         org   $7a00
         put   $3a00
*
noexec   equ   $10							* inhibit exec after loading (0 = NO exec)
dimask 	equ   $50							* mask to disable interrupts(FIRQ and IRQ)
eimask 	equ   $af                    	* mask to enable interrupts*
dnum   	equ   $12							* assigned drive number by DW4 server
numsec   equ   $76							* sector to be read (2 bytes)
nambuf 	equ   $1d1							* fixed by BASIC, read filename function which stores in $1d1
ramini 	equ   $0c00							* where to copy the 90 data sectors of each pack
linbuf	equ	$0d00							* buffer to contain the whole basic line
endbuf	equ   $0e00							* first byte out of the line buffer
syserr 	equ   $8344							* Show error message (B=error code)
okprmt   equ   $8371							* Basic OK prompt, command loop
outstr 	equ   $90e5							* print string at X+1 to devnum
getnam 	equ   $b7aa							* read file name and length into $1d1
ioerr  	equ   $b84b							* IO ERROR message
*
entry  	bsr   msg01 					   * print initial message
       	ldx   ,s								* get stack pointer to reg 'X'
       	ldu   1,x							* get 1ST address from vector table
       	stu   dwrvec  						* save as vector for DWRead
       	ldu   3,x							* get 2nd address from vector table
       	stu   dwwvec						* save as vector for DWWrite
       	ldu   5,x							* get 3rd address from vector table
       	stu   dwdvec						* save as vector for DWDoRead
*
       	orcc  #dimask			        	* disable FIRQ, IRQ (to work with DW4)
		 	jsr   <$a5							* peek at next character after dweeb name
       	suba  #'N							* to detect if it was an 'N' - if so don't exec it after loading
       	sta   <noexec						* save difference. 0 = noexec
       	bne   entnam						* if not 'N' skip next instruction
       	jsr   <$9f							* jump over that char (the 'N')
entnam 	jsr   getnam						* read file name and length into $1d1 = nambuf
       	ldx   #nambuf-1					* byte before packet buffer start
       	clr   ,x								* zero MSB to make a 16 bit value for Y
       	ldy   ,x++							* length of file name (16 bit)
       	beq   failed						* if len=0 no file entered
       	ldb   -1,x							* length of name (8 bit)
			clr   b,x							* zero terminate name string (for error)
       	inc   ,--x							* 1 = DriveWire OP_NAMEOBJ_MOUNT command
       	leay  2,y							* length of DW packet = name length + 2
       	jsr   (dwwvec)						* send file request to server
       	ldx   #dnum							* Our drive number variable
       	clr   ,x								* clear it
       	leay  1,y							* read one byte back
       	jsr   (dwrvec)						* get drive number
       	tst   ,x								* verify received value
       	bne   rascii						* successful mount. If Z has not been set = Error (coundn't mount)
       	ldb   #$19							* error code for MO error
		 	andcc #eimask       				* enable interrupts
       	jmp   syserr						* show error message
*
problm   andcc #eimask						* enable interrupts
			ldx   #toolng-1					* point to message
			fcb	#$10							* to skip next one
msg01  	ldx   #ldtext-1					* point to intro message
       	jsr   outstr						* print message to screen
		 	rts									* return
*
rascii  	clra									* zero to
			clrb									* 16 bits zero
			std	<numsec						* first sector to be requested
			bsr   redsec						* read a sector
			ldx   #ramini						* point to beginning of received data
			ldu   #linbuf						* point to basic line buffer
loop1		lda	,x+							* get a received char
			cmpa	#$0d							* is it an Enter?
			beq   sndlin						* yes, send line to interpreter
			sta   ,u+							* pass to line buffer
			cmpu  #endbuf						* got out of the line buffer?
			bhs   problm						* yes, we have a line too long!!
loop2		cmpx  #linbuf						* no, reached end of received data?
			blo	loop1							* no, process next char
			bsr   redsec						* yes, read another sector to complete partial sentence read
			ldx   #ramini						* point to beginning of received data
			lda   ,x								* get first char of this new sector
			cmpa  #$0a							* is it a line feed?
			bne   loop1							* no, process new data
			leax  1,x							* discard byte (tied to previous $0D)
			bra   loop1							* go process new data
*
failed 	andcc #eimask       				* enable interrupts
 		 	jmp   ioerr							* I/O Error message
*
sndlin	clr   ,u+							* put null to the
			clr   ,u								* two next bytes in buffer
			lda   ,x								* get next char
			cmpa  #$0a							* is it a line feed?
			bne   sndli1						* no, skip next
			leax  1,x							* increment pointer
sndli1	pshs  x,u							* save registers		
			ldu   #bckup						* point to stack backup area
			ldb   #6 							* first word is 6 bytes up in stack
loop3		ldx	b,s							* get word
			stx   ,u++							* save in backup
			subb  #2								* next word
			bge   loop3							* not finished, keep on saving
*
			ldx	#linbuf						* get linbuf address
			stx	<$a6							* pass to interpreter pointer
			jsr	<$a5							* get first char and verify for number
        	bsr   L83a3							* call input routine from interpreter
retinp	puls	x,u  							* restore pointers
			lda	,x								* get next byte to be read
			cmpa  #'0							* compare to '0'
			blo   final							* if lower, end
			cmpa	#'9							* compare to '9'
			bhi	final							* if higher, end
			ldu   #linbuf						* point to beginning of output buffer
			clra									* prepare a
			clrb									* 16 bits null
loop4		std	,u++							* clean a word from buffer
			cmpu	#endbuf						* got to the end of buffer?
			blo   loop4							* no, keep on cleaning
			ldu	#linbuf						* point to beginning again
			bra   loop2							* keep on reading input data
*
final  	andcc #eimask       				* enable interrupts
			leas	4,s							* clear stack
       	tst   <noexec						* has to be run?
       	beq   donot							* no skip it
			jsr   $841f							* reset basic vectors
			jsr   $849f							* run it
donot   	jmp   okprmt       				* return to interpreter
*
redsec	pshs  d,x,u							* save registers
			ldx   <numsec						* begin at sector zero
       	ldy   #ramini						* destination address
      	jsr   (dwdvec)						* read sector (into Y) 
       	bcs   failed						* detect frame error
       	bne   failed						* detect incomplete read
       	leax  1,X							* increment sector number
       	stx   <numsec						* save next sector number to be read
       	puls	d,x,u,pc						* restore registers and return
*
L83a3    jsr   $869a	   					* calculate line number   
L83a6    ldx   <$2b							* get line number
L83a8    stx   $02da							* puts it in front of text input buffer
L83ab    jsr   $8f67							* process / convert rest of the line
L83ae    stb   <$03							* save length of line
L83b0    jsr   $83ff							* search line in program
L83b2    bcs   L83C6							* not found, don't clear anything
L83b4    ldd   <$47							* line address								     
L83b6    subd  ,x								* - adress of next line					
L83b8    addd  <$1b							* + old start of variables				
L83ba    std   <$1b							* save as new start for variables	
L83bc    ldu   ,x								* Program from start of line			
L83be    pulu  a								* move down: get a byte						
L83c0    sta   ,x+							* put it at destination					
L83c2    cmpx  <$1b							* got to start of variables?			
L83c4    bne   L83be							* no, keep moving bytes					
L83c6    lda   $02dc							* gets 1st byte received
L83c9    beq   L83e7							* null? none received
L83cb    ldd   <$1b							* start for variables
L83cd    std   <$43							* save as old block
L83cf    addb  <$03							* add length of received line
L83d1    adca  #$00							* affect high byte
L83d3    std   <$41							* end of moved block
L83d5    jsr   $831c							* move the block
L83d8    ldu   #$02d8						* start of text - 4 (for link and line number)
L83db    pulu  a								* text from buffer
L83dd    sta   ,x+							* insert in program
L83df    cmpx  <$45							* got to end?
L83e1    bne   L83db							* not, keep on
L83e3    ldx   <$41							* end of moved block
L83e5    stx   <$1b							* put at start of variables
L83e7    jsr   $841f							* clear variables ...
L83e9    jsr   $83ed							* rebuild basic lines (re-link them)
L83eb		ldb	#5								* 5 words to push
			ldu   #bckup						* point to stack backup area
loop5		ldx   ,u++							* get a word
			pshs  x								* push onto stack
			decb									* decrement counter
			bne   loop5							* if not zero, keep on
			rts									* return
*
bckup    fdb   0,0,0,0,#retinp			* 5 words from stack
toolng	fcc	/LINE TOO LONG/,13,0    * error message

ldtext 	fcc   /ASCII LOADER/,13,0    	* message to user
dwrvec 	FDB   0								* indirect jump to dwload Read function
dwwvec 	FDB   0								* indirect jump to dwload Write function
dwdvec 	FDB   0								* indirect jump to dwload DoRead function
* 
